@@ -1,7 +1,28 @@
 make_EPSG <- function(file) {
-        if (missing(file)) {
+        if (.Call("PROJ4VersionInfo", PACKAGE = "rgdal")[[2]] >= 600L &&
+            !missing(file)) {
+            warning("New proj interface does not use file argument")
+            file <- NULL
+        }
+        metadata <- NULL
+        if (missing(file) || is.null(file)) {
             tf <- tempfile()
             n <- .Call("PROJcopyEPSG", tf, PACKAGE="rgdal")
+            if (PROJis6ormore()) {
+                if (n <= 0) stop("PROJ 6 database empty")
+                EPSG <- read.csv(tf, header=TRUE, stringsAsFactors=FALSE)
+                if (requireNamespace("DBI", quietly=TRUE) && 
+                    requireNamespace("RSQLite", quietly=TRUE)) {
+                    shpr <- get_proj_search_paths()
+                    db <- DBI::dbConnect(RSQLite::SQLite(),
+                        dbname=file.path(shpr[length(shpr)], "proj.db"))
+                    md <- DBI::dbReadTable(db, "metadata")
+                    DBI::dbDisconnect(db)
+                    metadata <- md[md$key == "EPSG.VERSION", "value"]
+                }
+                attr(EPSG, "metadata") <- metadata
+                return(EPSG)
+            }
             if (n > 0) file <- tf
             else stop("Error opening epsg file")
        } else {
@@ -20,7 +41,6 @@ make_EPSG <- function(file) {
 	note <- character(n)
 	prj4 <- character(n)
 	code <- integer(n)
-        metadata <- NULL
         if (!pre5) metadata <- readLines(epsg, n=1)
 	for (i in seq(along=code)) { # FIXME - broken in 5.0.0
 		res <- strsplit(paste(readLines(epsg, n=2), collapse=""), 
