@@ -255,7 +255,12 @@ SEXP get_source_crs(SEXP source) {
 
     PROTECT(res = NEW_CHARACTER(1));
     SET_STRING_ELT(res, 0,
-        COPY_TO_USER_STRING(proj_as_wkt(ctx, target_crs, PJ_WKT2_2019, NULL)));
+#if PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 3
+        COPY_TO_USER_STRING(proj_as_wkt(ctx, target_crs, PJ_WKT2_2018, NULL))
+#else
+        COPY_TO_USER_STRING(proj_as_wkt(ctx, target_crs, PJ_WKT2_2019, NULL))
+#endif
+    );
     UNPROTECT(1);
     proj_destroy(target_crs);
     proj_destroy(source_crs);
@@ -268,6 +273,11 @@ SEXP get_source_crs(SEXP source) {
 
 SEXP proj_vis_order(SEXP wkt2) {
 
+#if PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 3
+    warning("no CRS normalization available before PROJ 6.3");
+    return(wkt2);
+#else
+
     PJ_CONTEXT *ctx = proj_context_create();
     PJ *source_crs, *target_crs;
     SEXP res;
@@ -278,9 +288,7 @@ SEXP proj_vis_order(SEXP wkt2) {
         proj_context_destroy(ctx);
         error("proj_vis_order: source crs not created");
     }
-
     target_crs = proj_normalize_for_visualization(ctx, source_crs);
-
     if (target_crs == NULL) {
         proj_context_destroy(ctx);
         error("proj_vis_order: target crs not created");
@@ -288,14 +296,19 @@ SEXP proj_vis_order(SEXP wkt2) {
 
     PROTECT(res = NEW_CHARACTER(1));
     SET_STRING_ELT(res, 0,
-        COPY_TO_USER_STRING(proj_as_wkt(ctx, target_crs, PJ_WKT2_2019, NULL)));
+#if PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 3
+        COPY_TO_USER_STRING(proj_as_wkt(ctx, target_crs, PJ_WKT2_2018, NULL))
+#else
+        COPY_TO_USER_STRING(proj_as_wkt(ctx, target_crs, PJ_WKT2_2019, NULL))
+#endif
+    );
     UNPROTECT(1);
     proj_destroy(target_crs);
     proj_destroy(source_crs);
     proj_context_destroy(ctx);
 
     return(res);
-
+#endif
 }
 
 // unname(sapply(o[[2]], function(x) gsub(" ", " +", paste0("+", x))))
@@ -306,7 +319,7 @@ SEXP list_coordinate_ops(SEXP source, SEXP target, SEXP area_of_interest, SEXP s
     PJ_OPERATION_FACTORY_CONTEXT* operation_factory_context = NULL;
     PJ_OBJ_LIST *pj_operations = NULL;
     PJ *source_crs, *target_crs;
-    PJ* pj_transform = NULL;
+//    PJ* pj_transform = NULL;
     SEXP ans, input;
     int num_operations, i, j, is_instantiable, is_ballpark, grid_count;
     int pc=0;
@@ -315,6 +328,7 @@ SEXP list_coordinate_ops(SEXP source, SEXP target, SEXP area_of_interest, SEXP s
     const char *out_short_name, *out_full_name, *out_package_name, *out_url;
     PJ_PROJ_INFO pjinfo;
 
+// valgrind 210120
     source_crs = proj_create(ctx, CHAR(STRING_ELT(source, 0)));
 
     if (source_crs == NULL) {
@@ -322,6 +336,7 @@ SEXP list_coordinate_ops(SEXP source, SEXP target, SEXP area_of_interest, SEXP s
         error("source crs not created");
     }
     
+// valgrind 210120
     target_crs = proj_create(ctx, CHAR(STRING_ELT(target, 0)));
 
     if (target_crs == NULL) {
@@ -359,6 +374,7 @@ SEXP list_coordinate_ops(SEXP source, SEXP target, SEXP area_of_interest, SEXP s
             ctx, operation_factory_context,
             PROJ_GRID_AVAILABILITY_USED_FOR_SORTING);
 
+// valgrind 210120
     pj_operations = proj_create_operations(ctx,
                 source_crs, target_crs, operation_factory_context);
     
@@ -398,9 +414,16 @@ SEXP list_coordinate_ops(SEXP source, SEXP target, SEXP area_of_interest, SEXP s
 
 
     for (i=0; i<num_operations; i++) {
-        pj_transform = proj_list_get(ctx, pj_operations, i);
-        if (LOGICAL_POINTER(vis_order)[0])
-            pj_transform = proj_normalize_for_visualization(ctx, pj_transform);
+        PJ * pj_transform = proj_list_get(ctx, pj_operations, i);
+#if PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 1
+        warning("no Coordinate Operation normalization available before PROJ 6.1");
+#else
+        if (LOGICAL_POINTER(vis_order)[0]) {
+            PJ* pj_trans_orig = pj_transform;
+            pj_transform = proj_normalize_for_visualization(ctx, pj_trans_orig);
+            proj_destroy(pj_trans_orig);
+        }
+#endif
         is_instantiable = proj_coordoperation_is_instantiable(ctx,
             pj_transform);
         is_ballpark = proj_coordoperation_has_ballpark_transformation(ctx,
@@ -410,6 +433,7 @@ SEXP list_coordinate_ops(SEXP source, SEXP target, SEXP area_of_interest, SEXP s
         grid_count = proj_coordoperation_get_grid_used_count(ctx,
             pj_transform);
         pjinfo = proj_pj_info(pj_transform);
+
         SET_STRING_ELT(VECTOR_ELT(ans, 0), i,
             COPY_TO_USER_STRING(pjinfo.description));
         SET_STRING_ELT(VECTOR_ELT(ans, 1), i,
@@ -469,9 +493,12 @@ SEXP list_coordinate_ops(SEXP source, SEXP target, SEXP area_of_interest, SEXP s
                  } 
             }
         }
+        
+        proj_destroy(pj_transform);
+
     }
 
-    proj_destroy(pj_transform);
+//    proj_destroy(pj_transform);
     proj_list_destroy(pj_operations);
     proj_operation_factory_context_destroy(operation_factory_context);
     proj_destroy(source_crs); proj_destroy(target_crs);
@@ -576,8 +603,6 @@ SEXP transform_ng(SEXP fromargs, SEXP toargs, SEXP coordOp, SEXP npts, SEXP x, S
             //proj_context_destroy(ctx);
 	    error("coordinate operation creation failed: %s", errstr);
         }
-//        pj_transform = proj_normalize_for_visualization(ctx, pj_transform);
-
     } else {
 //Rprintf("source crs input: %s\n", CHAR(STRING_ELT(fromargs, 0)));
     	if ((source_crs = proj_create(PJ_DEFAULT_CTX, CHAR(STRING_ELT(fromargs, 0)))) == NULL) {
@@ -623,9 +648,15 @@ SEXP transform_ng(SEXP fromargs, SEXP toargs, SEXP coordOp, SEXP npts, SEXP x, S
         }
 #endif
 //Rprintf("%s\n", proj_pj_info(pj_transform).definition);
-        if (vis_order == 1) 
-            pj_transform = proj_normalize_for_visualization(PJ_DEFAULT_CTX, pj_transform);
-
+#if PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 1
+    warning("no Coordinate Operation normalization available before PROJ 6.1");
+#else
+        if (vis_order == 1) {
+            PJ* pj_trans_orig = pj_transform;
+            pj_transform = proj_normalize_for_visualization(PJ_DEFAULT_CTX, pj_trans_orig);
+            proj_destroy(pj_trans_orig);
+        }
+#endif
     }
 //Rprintf("%s\n", proj_pj_info(pj_transform).definition);
 //	if (proj_normalize_for_visualization(ctx, pj_transform) != NULL) // EJP
@@ -846,7 +877,13 @@ SEXP project_ng_coordOp(SEXP proj, SEXP inv, SEXP aoi, SEXP ob_tran
         error("coordinate operation creation failed: %s", errstr);
     }
 //Rprintf("%s\n", proj_pj_info(pj_transform).definition);
-    pj_transform = proj_normalize_for_visualization(PJ_DEFAULT_CTX, pj_transform);
+#if PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 1
+    warning("no Coordinate Operation normalization available before PROJ 6.1");
+#else
+    PJ* pj_trans_orig = pj_transform;
+    pj_transform = proj_normalize_for_visualization(PJ_DEFAULT_CTX, pj_trans_orig);
+    proj_destroy(pj_trans_orig);
+#endif
 //Rprintf("%s\n", proj_pj_info(pj_transform).definition);
 
     SEXP res;
@@ -934,36 +971,45 @@ SEXP P6_SRID_proj(SEXP inSRID, SEXP format, SEXP multiline, SEXP in_format,
     else if (LOGICAL_POINTER(enforce_xy)[0] == 0) vis_order = 0;
     else vis_order = 0;
 
-    PJ_CONTEXT *ctx = proj_context_create();
-    PJ *source_crs;
-    PJ_TYPE type;
-
-    if ((source_crs = proj_create(ctx, CHAR(STRING_ELT(inSRID, 0)))) == NULL) {
-        const char *errstr = proj_errno_string(proj_context_errno(ctx));
-        proj_context_destroy(ctx);
+    PJ *source_crs = proj_create(PJ_DEFAULT_CTX, CHAR(STRING_ELT(inSRID, 0)));
+    // valgrind resolved by copying EJP 
+    // https://github.com/r-spatial/gstat/issues/82#issuecomment-762970800
+    if (source_crs == NULL) {
+        const char *errstr = proj_errno_string(proj_context_errno(PJ_DEFAULT_CTX));
+//        proj_context_destroy(ctx);
 	error("source crs creation failed: %s", errstr);
     }
 
+#if (PROJ_VERSION_MAJOR > 6 || (PROJ_VERSION_MAJOR == 6 && PROJ_VERSION_MINOR >= 1))
+    PJ_TYPE type;
     type = proj_get_type(source_crs);
 
     if (type == PJ_TYPE_BOUND_CRS) {
-        source_crs = proj_get_source_crs(ctx, source_crs);
-
+        PJ *orig_crs = source_crs;
+        source_crs = proj_get_source_crs(PJ_DEFAULT_CTX, orig_crs);
+        proj_destroy(orig_crs);
         if (source_crs == NULL) {
-            proj_context_destroy(ctx);
+//            proj_context_destroy(ctx);
             error("crs not converted to source only");
         }
     }
+#endif
 
+#if (PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 3)
+    warning("no CRS normalization available before PROJ 6.3");
+#else
     if (vis_order) {
-        source_crs = proj_normalize_for_visualization(ctx, source_crs);
+        PJ *orig_crs = source_crs;
+        source_crs = proj_normalize_for_visualization(PJ_DEFAULT_CTX, orig_crs);
+        proj_destroy(orig_crs);
         if (source_crs == NULL) {
-            proj_context_destroy(ctx);
+//            proj_context_destroy(ctx);
             error("crs not converted to visualization order");
         }
     }
+#endif
 // FIXME PROJ 8.0 datum ensemble vulnerability
-    PJ* dtm = proj_crs_get_datum(ctx, source_crs);
+    PJ* dtm = proj_crs_get_datum(PJ_DEFAULT_CTX, source_crs);
     if (dtm != NULL) {
         PROTECT(Datum = NEW_CHARACTER(1)); pc++;
         SET_STRING_ELT(Datum, 0, COPY_TO_USER_STRING(proj_get_name(dtm)));
@@ -972,7 +1018,7 @@ SEXP P6_SRID_proj(SEXP inSRID, SEXP format, SEXP multiline, SEXP in_format,
         Datum = R_NilValue;
     }
 
-    PJ* ellps = proj_get_ellipsoid(ctx, source_crs);
+    PJ* ellps = proj_get_ellipsoid(PJ_DEFAULT_CTX, source_crs);
     if (ellps != NULL) {
         PROTECT(Ellps = NEW_CHARACTER(1)); pc++;
         SET_STRING_ELT(Ellps, 0, COPY_TO_USER_STRING(proj_get_name(ellps)));
@@ -987,9 +1033,14 @@ SEXP P6_SRID_proj(SEXP inSRID, SEXP format, SEXP multiline, SEXP in_format,
 
     if (INTEGER_POINTER(out_format)[0] == 1L) {
         
-        if ((pszSRS = proj_as_wkt(ctx, source_crs, PJ_WKT2_2019, NULL))
+#if PROJ_VERSION_MAJOR == 6  && PROJ_VERSION_MINOR < 3
+        if ((pszSRS = proj_as_wkt(PJ_DEFAULT_CTX, source_crs, PJ_WKT2_2018, NULL))
             == NULL) {
-            const char *errstr = proj_errno_string(proj_context_errno(ctx));
+#else
+        if ((pszSRS = proj_as_wkt(PJ_DEFAULT_CTX, source_crs, PJ_WKT2_2019, NULL))
+            == NULL) {
+#endif
+            const char *errstr = proj_errno_string(proj_context_errno(PJ_DEFAULT_CTX));
             warning("export to WKT2 failed: %s", errstr);
             SET_STRING_ELT(ans, 0, NA_STRING);
 	} else {
@@ -997,9 +1048,9 @@ SEXP P6_SRID_proj(SEXP inSRID, SEXP format, SEXP multiline, SEXP in_format,
         }
     } else if (INTEGER_POINTER(out_format)[0] == 2L) {
         
-        if ((pszSRS = proj_as_proj_string(ctx, source_crs, PJ_PROJ_4, NULL)) 
+        if ((pszSRS = proj_as_proj_string(PJ_DEFAULT_CTX, source_crs, PJ_PROJ_4, NULL)) 
             == NULL) {
-            const char *errstr = proj_errno_string(proj_context_errno(ctx));
+            const char *errstr = proj_errno_string(proj_context_errno(PJ_DEFAULT_CTX));
             warning("export to PROJ failed: %s", errstr);
             SET_STRING_ELT(ans, 0, NA_STRING);
 	} else {
@@ -1007,7 +1058,7 @@ SEXP P6_SRID_proj(SEXP inSRID, SEXP format, SEXP multiline, SEXP in_format,
         }
     } else {
         proj_destroy(source_crs);
-        proj_context_destroy(ctx);
+//        proj_context_destroy(ctx);
         UNPROTECT(pc);
         error("unknown output format");
     }
@@ -1016,7 +1067,7 @@ SEXP P6_SRID_proj(SEXP inSRID, SEXP format, SEXP multiline, SEXP in_format,
 
 
     proj_destroy(source_crs);
-    proj_context_destroy(ctx);
+//    proj_context_destroy(ctx);
     UNPROTECT(pc);
 
     return(ans);
@@ -1039,12 +1090,12 @@ PROJcopyEPSG(SEXP tf) {
     INTEGER_POINTER(ans)[0] = 0;
     int i, crs_cnt;
     PROJ_CRS_INFO **proj_crs_info;
-    //PJ_CONTEXT *ctx = proj_context_create();
+    PJ_CONTEXT *ctx = proj_context_create();
     FILE *fptf;
 
 	
     crs_cnt = 0;
-    proj_crs_info = proj_get_crs_info_list_from_database(PJ_DEFAULT_CTX, "EPSG", NULL,
+    proj_crs_info = proj_get_crs_info_list_from_database(ctx, "EPSG", NULL,
         &crs_cnt);
     if (crs_cnt < 1) {
         UNPROTECT(1);
@@ -1057,25 +1108,28 @@ PROJcopyEPSG(SEXP tf) {
     }
     fprintf(fptf, "\"code\",\"note\",\"prj4\",\"prj_method\"\n");
 
-    PJ *pj = NULL;
+//    PJ *pj = NULL;
 // blocks error messages in this context
-    proj_log_func(PJ_DEFAULT_CTX, NULL, proj_logger);
+    proj_log_func(ctx, NULL, proj_logger);
     for (i = 0; i < crs_cnt; i++) {
         const char *proj_definition;
 
-        pj = proj_create_from_database(PJ_DEFAULT_CTX, proj_crs_info[i]->auth_name,
-            proj_crs_info[i]->code, PJ_CATEGORY_CRS, 0, NULL);
-        proj_definition = proj_as_proj_string(PJ_DEFAULT_CTX, pj, PJ_PROJ_5, NULL);
+        PJ* pj = proj_create_from_database(ctx,
+            proj_crs_info[i]->auth_name, proj_crs_info[i]->code,
+            PJ_CATEGORY_CRS, 0, NULL);
+        proj_definition = proj_as_proj_string(ctx, pj, // valgrind
+            PJ_PROJ_5, NULL);
+//        proj_destroy(pj); // valgrind
 
-        fprintf(fptf, "%s,\"%s\",\"%s\",\"%s\"\n", proj_crs_info[i]->code,
+        fprintf(fptf, "%s,\"%s\",\"%s\",\"%s\"\n", proj_crs_info[i]->code, //ASAN
   	    proj_crs_info[i]->name, proj_definition, 
             proj_crs_info[i]->projection_method_name);
+        proj_destroy(pj);
     }
 
     fclose(fptf);
-    proj_destroy(pj);
     proj_crs_info_list_destroy(proj_crs_info);
-    //proj_context_destroy(ctx);
+    proj_context_destroy(ctx);
     INTEGER_POINTER(ans)[0] = crs_cnt;
     UNPROTECT(1);
 
